
# q-expansion of newform 189.3.b.c, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 189
weight = 3
poly_data = [4, 0, 56, 0, 53, 0, 14, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 4, 0, 1, 0, 0, 0, 0], 1], [[-20, 0, 16, 0, 11, 0, 1, 0], 8], [[28, 0, 72, 0, 33, 0, 3, 0], 8], [[0, 36, 0, 42, 0, 13, 0, 1], 4], [[28, 0, 49, 0, 14, 0, 1, 0], 2], [[0, 40, 0, 118, 0, 39, 0, 3], 8], [[0, 260, 0, 272, 0, 71, 0, 5], 8]]

hecke_ring_character_values = [[29, [-1, 0, 0, 0, 0, 0, 0, 0]], [136, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -1, 0, -1, 1], [0, 0, -1, 0, 0, 0, 0, 0], [0, 1, 0, 0, -3, 0, 2, -2], [4, 0, 2, 1, 0, 2, 0, 0], [0, -1, 0, 0, -1, 0, -3, 0], [2, 0, -7, -1, 0, -1, 0, 0], [0, 3, 0, 0, 4, 0, 2, 0], [0, -2, 0, 0, -5, 0, -2, 4], [-2, 0, 4, -3, 0, -2, 0, 0], [-2, 0, -13, 3, 0, 0, 0, 0], [0, -2, 0, 0, 1, 0, 5, 3], [-15, 0, 6, -8, 0, 0, 0, 0], [0, -6, 0, 0, -9, 0, -3, 7], [0, -5, 0, 0, 4, 0, 6, -2], [0, -7, 0, 0, 5, 0, 3, -6], [24, 0, -3, -3, 0, 1, 0, 0], [-9, 0, 3, -15, 0, -2, 0, 0], [0, -1, 0, 0, 14, 0, 0, 10], [30, 0, 2, 8, 0, -6, 0, 0], [38, 0, 5, 3, 0, 4, 0, 0], [0, 10, 0, 0, -1, 0, 11, 1], [0, -9, 0, 0, 8, 0, -10, 1], [-52, 0, 7, 13, 0, -5, 0, 0], [0, -14, 0, 0, -10, 0, -20, 22], [-46, 0, 13, 18, 0, -1, 0, 0], [0, 16, 0, 0, -10, 0, 18, -2], [-38, 0, -7, -3, 0, 0, 0, 0], [0, 1, 0, 0, -43, 0, 12, -12], [26, 0, 25, 7, 0, -8, 0, 0], [0, -1, 0, 0, 0, 0, 0, -21], [0, -13, 0, 0, -41, 0, -20, 16], [140, 0, 23, 1, 0, 3, 0, 0], [0, 16, 0, 0, 15, 0, 22, 16], [-112, 0, 5, 3, 0, 16, 0, 0], [92, 0, 10, 17, 0, 0, 0, 0], [-87, 0, -10, -12, 0, -4, 0, 0], [0, 18, 0, 0, 51, 0, -17, 11], [0, 10, 0, 0, 58, 0, -10, 4], [0, -7, 0, 0, -23, 0, -12, 8], [12, 0, 27, -18, 0, 13, 0, 0], [0, 12, 0, 0, 34, 0, 30, -22], [-7, 0, 18, -18, 0, -6, 0, 0], [0, -27, 0, 0, 15, 0, -2, 8], [142, 0, -1, 12, 0, 15, 0, 0], [-17, 0, -49, 13, 0, -16, 0, 0], [-180, 0, 43, 51, 0, -3, 0, 0], [0, -17, 0, 0, 24, 0, -26, 23], [-88, 0, 68, -36, 0, 4, 0, 0], [0, 1, 0, 0, -9, 0, -44, 34], [0, -3, 0, 0, -43, 0, -44, -2], [-154, 0, -37, 27, 0, 19, 0, 0], [0, -2, 0, 0, 45, 0, 17, -13], [0, 2, 0, 0, -48, 0, 52, 2], [0, 28, 0, 0, -71, 0, -8, -36], [0, -4, 0, 0, -47, 0, 7, -17], [-18, 0, -35, 18, 0, 3, 0, 0], [-108, 0, -41, 3, 0, -24, 0, 0], [0, -22, 0, 0, -22, 0, -10, 2], [12, 0, -7, -15, 0, 17, 0, 0], [0, -8, 0, 0, -103, 0, -29, 5], [112, 0, -102, 10, 0, -20, 0, 0], [0, 42, 0, 0, 27, 0, 51, -43], [-86, 0, 0, -34, 0, 42, 0, 0], [0, -16, 0, 0, 30, 0, 26, -62], [175, 0, -12, -46, 0, 30, 0, 0], [-63, 0, 152, 24, 0, 14, 0, 0], [0, -21, 0, 0, 5, 0, 0, -14], [-248, 0, -7, 58, 0, 11, 0, 0], [0, 29, 0, 0, -143, 0, 53, -24], [0, 9, 0, 0, -44, 0, -2, -56], [-150, 0, 147, -22, 0, 5, 0, 0], [-170, 0, -17, 41, 0, -10, 0, 0], [-256, 0, -59, 25, 0, -26, 0, 0], [0, 34, 0, 0, -113, 0, -9, 15], [0, -5, 0, 0, 117, 0, -26, 32], [-104, 0, 34, 42, 0, 16, 0, 0], [0, -50, 0, 0, 164, 0, -76, 44], [-124, 0, 138, 12, 0, 6, 0, 0], [0, 15, 0, 0, -39, 0, 69, 12], [98, 0, 52, -42, 0, -22, 0, 0], [0, -20, 0, 0, -96, 0, 16, -4], [328, 0, 3, 55, 0, 1, 0, 0], [162, 0, -31, 8, 0, -51, 0, 0], [0, -55, 0, 0, 55, 0, -74, 0], [0, -45, 0, 0, -117, 0, 60, -72], [21, 0, -91, 9, 0, -10, 0, 0], [0, -62, 0, 0, 141, 0, -69, 73], [-12, 0, -33, -15, 0, -50, 0, 0], [0, -4, 0, 0, 202, 0, 64, 30], [0, 4, 0, 0, 85, 0, -91, 47], [-94, 0, 32, 62, 0, 62, 0, 0], [0, -57, 0, 0, -71, 0, 64, 0], [36, 0, 31, -9, 0, -6, 0, 0], [0, 14, 0, 0, 93, 0, -85, -3], [0, 54, 0, 0, 85, 0, 55, 19], [0, -45, 0, 0, 47, 0, 13, 48], [450, 0, 11, 15, 0, -37, 0, 0], [280, 0, 107, 41, 0, -8, 0, 0], [280, 0, 135, 31, 0, 4, 0, 0], [0, 35, 0, 0, -182, 0, 128, -60], [0, 33, 0, 0, -48, 0, 70, -23], [0, 82, 0, 0, -122, 0, 106, -88], [271, 0, -146, -16, 0, -4, 0, 0], [228, 0, 136, 12, 0, -24, 0, 0], [0, -31, 0, 0, 58, 0, 38, 91], [0, 76, 0, 0, 83, 0, 137, -133], [0, -14, 0, 0, 19, 0, -66, -14], [-310, 0, -163, -29, 0, -39, 0, 0], [-242, 0, -126, 47, 0, 0, 0, 0], [-174, 0, 112, -54, 0, 6, 0, 0], [0, -52, 0, 0, -70, 0, -126, 34], [-492, 0, -43, 87, 0, 17, 0, 0], [-240, 0, -31, -41, 0, 70, 0, 0], [0, 17, 0, 0, -11, 0, -14, 100], [18, 0, -306, 4, 0, 6, 0, 0], [0, -20, 0, 0, 116, 0, -104, 136], [0, 30, 0, 0, 103, 0, 20, -54], [0, -8, 0, 0, 116, 0, 80, -56], [-100, 0, -57, -121, 0, -35, 0, 0], [309, 0, -221, -81, 0, 0, 0, 0], [0, 54, 0, 0, 220, 0, 8, -12], [0, -33, 0, 0, -317, 0, -100, 28], [-106, 0, 175, -21, 0, -1, 0, 0], [0, 85, 0, 0, 37, 0, 36, -142], [22, 0, -175, 3, 0, -60, 0, 0], [0, -32, 0, 0, 187, 0, -27, -33], [366, 0, 70, -125, 0, -58, 0, 0], [388, 0, 89, 42, 0, -9, 0, 0], [-338, 0, 6, 54, 0, 20, 0, 0], [0, 36, 0, 0, -69, 0, 42, -78], [320, 0, -302, -10, 0, 18, 0, 0], [-282, 0, -21, -11, 0, -36, 0, 0], [0, -21, 0, 0, -103, 0, -113, -82], [-286, 0, 224, 30, 0, -38, 0, 0], [0, -58, 0, 0, -171, 0, 85, -99], [108, 0, 0, 90, 0, 22, 0, 0], [0, 10, 0, 0, -50, 0, 88, -32], [0, 6, 0, 0, 198, 0, -54, -70], [-256, 0, 165, 9, 0, 63, 0, 0], [0, 11, 0, 0, 10, 0, 0, 64], [-20, 0, -103, 45, 0, 6, 0, 0], [0, -93, 0, 0, -37, 0, -68, 184], [400, 0, 7, 23, 0, 47, 0, 0], [0, 106, 0, 0, 75, 0, 47, 101], [188, 0, 81, -42, 0, 99, 0, 0], [0, 9, 0, 0, -215, 0, -15, -108], [-604, 0, -110, 18, 0, -40, 0, 0], [0, 73, 0, 0, 108, 0, 118, -164], [300, 0, 199, 27, 0, -36, 0, 0], [0, -9, 0, 0, -346, 0, -74, -41], [-85, 0, -100, 30, 0, -74, 0, 0], [0, -54, 0, 0, -323, 0, 31, 19], [872, 0, 121, -3, 0, -22, 0, 0], [0, 123, 0, 0, 173, 0, 142, -20], [386, 0, 103, 45, 0, -40, 0, 0], [0, 0, 0, 0, 63, 0, 53, -133], [-694, 0, 123, 97, 0, -55, 0, 0], [0, 118, 0, 0, 181, 0, -139, 23], [0, 36, 0, 0, -192, 0, -52, 170], [0, 6, 0, 0, 230, 0, 126, -208], [-470, 0, -184, -134, 0, -106, 0, 0], [0, -133, 0, 0, -197, 0, -23, -66], [0, 48, 0, 0, 280, 0, 104, -98], [0, -10, 0, 0, 57, 0, 59, -175], [176, 0, -357, 3, 0, -84, 0, 0], [-852, 0, 68, -27, 0, 74, 0, 0]]
